﻿using System;
using System.Collections.Generic;
using System.Diagnostics;

using EA;
using WorkflowService;
using WorkflowService.Model.EntityClasses;

namespace EaWorkflowDesignerAddin.Ui
{
    public class MainformPresenter
    {
        private MainForm _view = new MainForm();

        public void ShowView(Repository repository)
        {
            if (_view == null || _view.IsDisposed)
            {
                _view = new MainForm();
            }

            _view.Presenter = new MainformPresenter();
            _view.Repository = repository;
            _view.ShowDialog();
        }

        public void GenerateWorkflowFromDiagram(
            string databaseServerName,
            string databaseName,
            Diagram stateMachineDiagram,
            Repository repository)
        {
            if (stateMachineDiagram.Type != "Statechart")
            {
                _view.ShowError("Can only generate a workflow from a Statechart");
            }
            WorkflowEntity workflowEntity = new WorkflowEntity();
            workflowEntity.Name = stateMachineDiagram.Name;
            workflowEntity.Guid = Guid.NewGuid();

            // Add all the states:
            foreach (DiagramObject diagramObject in stateMachineDiagram.DiagramObjects)
            {
                Element element = repository.GetElementByID(diagramObject.ElementID);
                Debug.Print("{0}: Type={1}, MetaType={2}", element.Name, element.Type, element.MetaType);
                if (element.Type == "State" || element.Type == "StateNode")
                {
                    StateEntity state = new StateEntity();
                    state.Name = element.Name;
                    if (element.Type == "StateNode" && element.MetaType == "Pseudostate")
                        state.IsInitialState = true;
                    if (element.Type == "StateNode" && element.MetaType == "FinalState")
                        state.IsFinalState = true;
                    workflowEntity.State.Add(state);
                }
            }

            // Add all the allowed transitions:
            foreach (DiagramLink diagramLink in stateMachineDiagram.DiagramLinks)
            {
                Connector connector = repository.GetConnectorByID(diagramLink.ConnectorID);
                if (connector.Type == "StateFlow")
                {
                    if (connector.MetaType == "Transition")
                    {
                        Element client = repository.GetElementByID(connector.ClientID);
                        Element supplier = repository.GetElementByID(connector.SupplierID);
                        string clientName = client.Name;
                        string supplierName = supplier.Name;
                        Debug.Print("{0} -> {1}", clientName, supplierName);
                        CanTransitionToEntity canTransitionToEntity = new CanTransitionToEntity();
                        StateEntity fromState = GetStateByNameFromList(workflowEntity.State, clientName);
                        StateEntity toState = GetStateByNameFromList(workflowEntity.State, supplierName);
                        canTransitionToEntity.FromState = fromState;
                        canTransitionToEntity.ToState = toState;
                        fromState.CanTransitionTo.Add(canTransitionToEntity);
                    }
                }
            }

            // now save the Workflow object graph to the database:
            try
            {
                IWorkflowService svc = new WorkflowServiceImpl(databaseServerName, databaseName);
                svc.SaveWorkflow(workflowEntity);
                svc.Terminate();
            }
            catch (Exception ex)
            {
                string msg = ex.Message;
                if (ex.InnerException != null) {
                    msg += Environment.NewLine +
                        Environment.NewLine +
                        ex.InnerException.Message;
                }
                _view.ShowError(msg);
            }
        }

        private StateEntity GetStateByNameFromList(IList<StateEntity> states, string stateName)
        {
            foreach (StateEntity state in states)
            {
                if (state.Name == stateName) return state;
            }
            return null;
        }


    }
}
